@@ -34,27 +34,33 @@
 //! - **Public Parameter** The parameter generated during setup. The users can use
 //! this parameter to prove their transaction validity. This parameter can be gotten
 //! throught RPC client.
-
 #![cfg_attr(not(feature = "std"), no_std)]
-#![allow(clippy::unused_unit)]
 
-use dusk_plonk::prelude::{Circuit, Proof, PublicInputValue, PublicParameters, VerifierData};
-use frame_support::pallet_prelude::*;
-use frame_system::pallet_prelude::*;
 pub use pallet::*;
-use parity_scale_codec::{Decode, Encode};
-use sp_std::vec::Vec;
+
+#[cfg(test)]
+mod mock;
 
 #[cfg(test)]
 mod tests;
 
+#[cfg(feature = "runtime-benchmarks")]
+mod benchmarking;
+
+mod traits;
+mod types;
+
+pub use traits::Plonk;
+pub use types::*;
+
+use frame_support::dispatch::{DispatchErrorWithPostInfo, PostDispatchInfo};
+use frame_support::pallet_prelude::*;
+use frame_system::pallet_prelude::*;
+use sp_std::vec::Vec;
+
 #[frame_support::pallet]
 pub mod pallet {
-    use super::{Circuit, Proof, PublicInputValue, PublicParameters, Transcript, VerifierData, Vec};
-    use frame_support::dispatch::{DispatchErrorWithPostInfo, PostDispatchInfo};
-    use frame_support::pallet_prelude::*;
-    use frame_system::pallet_prelude::*;
-    use rand_core::OsRng;
+    use super::*;
 
     #[pallet::config]
     pub trait Config: frame_system::Config {
@@ -77,6 +83,14 @@ pub mod pallet {
         TrustedSetup(PublicParameters),
     }
 
+    #[pallet::error]
+    pub enum Error<T> {
+        /// Error names should be descriptive.
+        NoneValue,
+        /// Errors should have helpful documentation associated with them.
+        StorageOverflow,
+    }
+
     #[pallet::pallet]
     #[pallet::generate_store(pub(super) trait Store)]
     pub struct Pallet<T>(_);
@@ -88,22 +102,14 @@ pub mod pallet {
     impl<T: Config> Pallet<T> {
         /// The function called when we setup the parameters
         #[pallet::weight(10_000)]
-        pub fn trusted_setup(origin: OriginFor<T>, val: u32) -> DispatchResultWithPostInfo {
-            let _ = ensure_signed(origin)?;
-            match Self::public_parameter() {
-                Some(_) => {
-                    return Err(DispatchErrorWithPostInfo {
-                        post_info: PostDispatchInfo::from(()),
-                        error: DispatchError::Other("already setup"),
-                    })
-                }
-                None => {
-                    let pp = PublicParameters::setup(1 << val, &mut OsRng).unwrap();
-                    PublicParameter::<T>::put(&pp);
-                    Event::<T>::TrustedSetup(pp);
-                    return Ok(().into());
-                }
-            }
+        pub fn trusted_setup(
+            origin: OriginFor<T>,
+            val: u32,
+            rng: ParityRng,
+        ) -> DispatchResultWithPostInfo {
+            let transactor = ensure_signed(origin)?;
+            <Self as Plonk<_>>::trusted_setup(&transactor, val, rng)?;
+            Ok(().into())
         }
 
         /// The function called when we verify the statement
@@ -115,32 +121,9 @@ pub mod pallet {
             public_inputs: Vec<PublicInputValue>,
             transcript_init: Transcript,
         ) -> DispatchResultWithPostInfo {
-            let _ = ensure_signed(origin)?;
-            match Self::public_parameter() {
-                Some(pp) => {
-                    match T::CustomCircuit::verify(
-                        &pp,
-                        &vd,
-                        &proof,
-                        &public_inputs,
-                        transcript_init.0,
-                    ) {
-                        Ok(_) => return Ok(().into()),
-                        Err(_) => {
-                            return Err(DispatchErrorWithPostInfo {
-                                post_info: PostDispatchInfo::from(()),
-                                error: DispatchError::Other("invalid proof"),
-                            })
-                        }
-                    }
-                }
-                None => {
-                    return Err(DispatchErrorWithPostInfo {
-                        post_info: PostDispatchInfo::from(()),
-                        error: DispatchError::Other("setup not yet"),
-                    })
-                }
-            }
+            let transactor = ensure_signed(origin)?;
+            <Self as Plonk<_>>::verify(&transactor, vd, proof, public_inputs, transcript_init)?;
+            Ok(().into())
         }
     }
 }
@@ -149,27 +132,58 @@ impl<T: Config> Pallet<T> {
     pub fn get_public_parameters() -> Option<PublicParameters> {
         PublicParameter::<T>::get()
     }
+}
 
-    pub fn verify_proof(
-        origin: OriginFor<T>,
+impl<T: Config> Plonk<T::AccountId> for Pallet<T> {
+    type CustomCircuit = T::CustomCircuit;
+
+    fn trusted_setup(
+        _who: &T::AccountId,
+        val: u32,
+        mut rng: ParityRng,
+    ) -> DispatchResultWithPostInfo {
+        match Self::public_parameter() {
+            Some(_) => {
+                return Err(DispatchErrorWithPostInfo {
+                    post_info: PostDispatchInfo::from(()),
+                    error: DispatchError::Other("already setup"),
+                })
+            }
+            None => {
+                let pp = PublicParameters::setup(1 << val, &mut rng).unwrap();
+                PublicParameter::<T>::put(&pp);
+                Event::<T>::TrustedSetup(pp);
+                return Ok(().into());
+            }
+        }
+    }
+
+    fn verify(
+        _who: &T::AccountId,
         vd: VerifierData,
         proof: Proof,
         public_inputs: Vec<PublicInputValue>,
         transcript_init: Transcript,
     ) -> DispatchResultWithPostInfo {
-        Self::verify(origin, vd, proof, public_inputs, transcript_init)
-    }
-}
-
-/// The struct for Merlin transcript
-#[derive(Debug, PartialEq, Clone, Encode)]
-pub struct Transcript(pub &'static [u8]);
-
-#[allow(unconditional_recursion)]
-impl Decode for Transcript {
-    fn decode<I: parity_scale_codec::Input>(
-        input: &mut I,
-    ) -> Result<Self, parity_scale_codec::Error> {
-        Decode::decode(input)
+        match Self::public_parameter() {
+            Some(pp) => {
+                match T::CustomCircuit::verify(&pp, &vd, &proof, &public_inputs, transcript_init.0)
+                {
+                    Ok(_) => return Ok(().into()),
+                    Err(_) => {
+                        return Err(DispatchErrorWithPostInfo {
+                            post_info: PostDispatchInfo::from(()),
+                            error: DispatchError::Other("invalid proof"),
+                        })
+                    }
+                }
+            }
+            None => {
+                return Err(DispatchErrorWithPostInfo {
+                    post_info: PostDispatchInfo::from(()),
+                    error: DispatchError::Other("setup not yet"),
+                })
+            }
+        }
     }
 }
